import os
import sys
import string
import time
import pickle
import textwrap
import sqlite3
import copy
from datetime import datetime

def prepare_status_db(conn):
    """
    Creates table of experiment status if it does not exist
    :return: connection object and current contents of table.
    :rtype: Tuple
    """
    create_table_query = textwrap.dedent(
    '''CREATE TABLE
    IF NOT EXISTS experiment_status (
    exp_id integer PRIMARY KEY,
    name text NOT NULL,
    status text NOT NULL,
    seconds_diff integer NOT NULL,
    modified text NOT NULL,
    FOREIGN KEY (exp_id) REFERENCES experiment (id)
    );''')
    drop_table_query = ''' DROP TABLE experiment_status '''
    # create_table(conn, drop_table_query)
    create_table(conn, create_table_query)    
    current_table = _get_exp_status(conn)
    current_table_expid = dict()
    # print(current_table)
    # print(type(current_table))
    for item in current_table:
        exp_id, expid, status, seconds = item
        current_table_expid[expid] = (exp_id, status, seconds)
    return current_table_expid

def prepare_completed_times_db(conn):
    """
    Creates two tables: 
    "experiment_times" that stores general information about experiments' jobs completed updates
    "job_times" that stores information about completed time for jobs    
    """
    create_table_header_query = textwrap.dedent(
    '''CREATE TABLE
    IF NOT EXISTS experiment_times (
    exp_id integer PRIMARY KEY,
    name text NOT NULL,
    created int NOT NULL,    
    modified int NOT NULL,
    total_jobs int NOT NULL,
    completed_jobs int NOT NULL,
    FOREIGN KEY (exp_id) REFERENCES experiment (id)
    );''')

    create_table_detail_query = textwrap.dedent(
    '''CREATE TABLE
    IF NOT EXISTS job_times (
    detail_id integer PRIMARY KEY AUTOINCREMENT,
    exp_id integer NOT NULL,
    exp_name text NOT NULL,
    job_name text NOT NULL,    
    created integer NOT NULL,    
    modified integer NOT NULL,
    submit_time int NOT NULL,
    start_time int NOT NULL,
    finish_time int NOT NULL,
    status text NOT NULL,
    FOREIGN KEY (exp_id) REFERENCES experiment (id)
    );''')

    drop_table_header_query = ''' DROP TABLE experiment_times '''
    drop_table_details_query = ''' DROP TABLE job_times '''
    # create_table(conn, drop_table_details_query)
    # create_table(conn, drop_table_header_query)
    # return
    create_table(conn, create_table_header_query)  
    create_table(conn, create_table_detail_query)  
    current_table = _get_exp_times(conn)
    current_table_expid = dict()
    for item in current_table:
        exp_id, name, created, modified, total_jobs, completed_jobs = item
        current_table_expid[name] = (exp_id, int(created), int(modified), int(total_jobs), int(completed_jobs))
    return current_table_expid

def get_times_detail(conn, expid):
    """
    Gets the current detail of the experiment from job_times
    """
    current_table_detail = dict()
    current_table = _get_job_times(conn, expid)
    for item in current_table_detail:
        exp_id, exp_name, job_name, created, modified, minutes, status = item
        current_table_detail[job_name] = (created, modified, minutes, status)
    return current_table_detail
        

def insert_experiment_times_header(conn, expid, timest, total_jobs, completed_jobs):
    """
    Inserts into experiment times header
    """
    current_id = _get_id_db(conn, expid)
    if (current_id):
        row_content = (current_id, expid, timest, timest, total_jobs, completed_jobs)
        result = _create_exp_times(conn, row_content)
        return current_id
    else:
        return -1

def _create_exp_times(conn, row_content):
    """
    Create experiment times
    :param conn:
    :param details:
    :return:
    """
    try:
        sql = ''' INSERT INTO experiment_times(exp_id, name, created, modified, total_jobs, completed_jobs) VALUES(?,?,?,?,?,?) '''
        # print(row_content)
        cur = conn.cursor()
        cur.execute(sql, row_content)        
        # print(cur)
        conn.commit()
        return cur.lastrowid
    except sqlite3.Error as e:
        print("Error on Insert : " + str(type(e).__name__))
        print(row_content)

def create_job_times(conn, exp_id, exp_name, job_name, timest, submit_time, start_time, finish_time, status):
    """
    Create experiment times
    :param conn:
    :param details:
    :return:
    """
    try:
        sql = ''' INSERT INTO job_times(exp_id, exp_name, job_name, created, modified, submit_time, start_time, finish_time, status) VALUES(?,?,?,?,?,?,?,?,?) '''
        # print(row_content)
        cur = conn.cursor()
        cur.execute(sql, (exp_id, exp_name, job_name, timest, timest, submit_time, start_time, finish_time, status))        
        # print(cur)
        conn.commit()
        return cur.lastrowid
    except sqlite3.Error as e:
        print("Error on Insert : " + str(type(e).__name__))
        print((exp_id, exp_name, job_name, timest, timest, submit_time, start_time, finish_time, status))



def insert_experiment_status(conn, expid, seconds_diff):
    """
    Inserts into table of status if it does not exist and updates status
    """            
    current_id = _get_id_db(conn, expid)
    if (current_id):
        row_content = (current_id, expid, "RUNNING", int(seconds_diff), datetime.today().strftime('%Y-%m-%d-%H:%M:%S'))
        # print(row_content)
        result = _create_exp_status(conn, row_content)     
    return current_id   

def create_connection(db_file):
    """ 
    create a database connection to the SQLite database specified by db_file
    :param db_file: database file
    :return: Connection object or None
    """
    try:
        conn = sqlite3.connect(db_file)
        return conn
    except:
        return None

def create_table(conn, create_table_sql):
    """ create a table from the create_table_sql statement
    :param conn: Connection object
    :param create_table_sql: a CREATE TABLE statement
    :return:
    """
    try:
        c = conn.cursor()
        c.execute(create_table_sql)
    except Exception as e:
        print(e)

def get_exps(conn):
    """
    Get exp ids
    :param conn:
    :param expid:
    :return:
    """
    # SQLite always return a unicode object, but we can change this
    # behaviour with the next sentence
    conn.text_factory = str
    cur = conn.cursor()
    cur.execute("SELECT name FROM experiment WHERE autosubmit_version IS NOT NULL")
    rows = cur.fetchall()
    return rows

def _get_exp_times(conn):
    """
    Get exp times header
    """
    conn.text_factory = str
    cur = conn.cursor()
    cur.execute("SELECT exp_id, name, created, modified, total_jobs, completed_jobs FROM experiment_times")
    rows = cur.fetchall()
    return rows

def _get_job_times(conn, expid):
    """
    Get exp job times detail for a given expid
    """
    cur = conn.cursor()
    cur.execute("SELECT exp_id, exp_name, job_name, created, modified, minutes, status FROM job_times WHERE exp_name=?", (expid,))
    rows = cur.fetchall()
    return rows

def _get_exp_status(conn):
    """
    Get exp with status
    """
    conn.text_factory = str
    cur = conn.cursor()
    cur.execute("SELECT exp_id, name, status, seconds_diff FROM experiment_status")
    rows = cur.fetchall()
    return rows

def _create_exp_status(conn, row_content):
    """
    Create experiment status
    :param conn:
    :param details:
    :return:
    """
    try:
        sql = ''' INSERT INTO experiment_status(exp_id, name, status, seconds_diff, modified) VALUES(?,?,?,?,?) '''
        # print(row_content)
        cur = conn.cursor()
        cur.execute(sql, row_content)        
        # print(cur)
        conn.commit()
        return cur.lastrowid
    except sqlite3.Error as e:
        print("Error on Insert : " + str(type(e).__name__))

def update_exp_status(conn, expid, status, seconds_diff):
    """
    Update existing experiment status
    """
    try:
        sql = ''' UPDATE experiment_status SET status = ?, seconds_diff = ?, modified = ? WHERE name = ? '''
        cur = conn.cursor()
        cur.execute(sql, (status, seconds_diff, datetime.today().strftime('%Y-%m-%d-%H:%M:%S'), expid))
        conn.commit()
        return cur.lastrowid
    except sqlite3.Error as e:
        print("Error on Update: " + str(type(e).__name__))
    

def _get_id_db(conn, expid):
    """
    Get exp id
    :param conn:
    :param expid:
    :return:
    """
    cur = conn.cursor()
    cur.execute("SELECT id FROM experiment WHERE name=?", (expid,))
    row = cur.fetchone()
    return row[0]