#!/usr/bin/env python

# Copyright 2015 Earth Sciences Department, BSC-CNS

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.

"""
Module containing functions to manage autosubmit's experiments.
"""
import os
import sys
import string
import time
import pickle
import textwrap
import sqlite3
import datetime
import copy

# try:
#     import tkinter
# except:
#     import Tkinter as tkinter


import autosubmitAPIwu.database.db_common as db_common
from autosubmitAPIwu.job.job_list import JobList
from autosubmitAPIwu.job.job import Job
from autosubmitAPIwu.job.job_package_persistence import JobPackagePersistence
from autosubmitAPIwu.job.job_grouping import JobGrouping
from autosubmitAPIwu.monitor.monitor import Monitor
from autosubmitAPIwu.job.job_common import Status, Type
from autosubmitAPIwu.experiment.statistics import ExperimentStats
import autosubmitAPIwu.experiment.common_db_requests as DbRequests
from autosubmitAPIwu.config.basicConfig import BasicConfig
from autosubmitAPIwu.config.config_common import AutosubmitConfig
from bscearth.utils.config_parser import ConfigParserFactory
from bscearth.utils.log import Log
from bscearth.utils.date import date2str
from autosubmitAPIwu.autosubmit import Autosubmit

def get_experiment_stats(expid, filter_period, filter_type):
    """
    Lite version of the stats generator from Autosubmit autosubmit.py
    """
    error = False
    error_message = ""
    period_fi = ""
    period_ini = ""
    otherstats = list()
    jobs = list()
    results = None
    try:  
        if filter_period:
            filter_period = int(filter_period)                
        BasicConfig.read()
        as_conf = AutosubmitConfig(expid, BasicConfig, ConfigParserFactory())
        if not as_conf.check_conf_files():
            raise Exception('Autosubmit GUI might not have permission to access necessary configuration files.')
        # notransitive as False by default
        job_list = Autosubmit.load_job_list(expid, as_conf, False)
        if filter_type:
            ft = filter_type
            if ft == 'Any':
                job_list = job_list.get_job_list()
            else:
                job_list = [job for job in job_list.get_job_list() if job.section == ft]
        else:
            ft = 'Any'
            job_list = job_list.get_job_list()

        period_fi = datetime.datetime.now().replace(second=0, microsecond=0)
        if filter_period and filter_period > 0:                     
            period_ini = period_fi - datetime.timedelta(hours=filter_period)
            job_list = [job for job in job_list if
                        job.check_started_after(period_ini) or job.check_running_after(period_ini)]
        else:
            period_ini = None
       
        if len(job_list) > 0:
            # Getting those predefined stats
            general_stats = []
            general_stats_path = os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, "tmp", expid + "_GENERAL_STATS")
            parser = AutosubmitConfig.get_parser(ConfigParserFactory(), general_stats_path)
            for section in parser.sections():
                general_stats.append((section, ''))
                general_stats += parser.items(section)
            otherstats = general_stats
            
            results = ExperimentStats(job_list, period_ini, period_fi)
            
            jobs = [job.name for job in results._jobs_list]
        else:
            raise Exception("There are no {0} jobs in the period from {1} to {2}...".format(ft, period_ini, period_fi))
        
    except Exception as e:
        error_message = str(e)
        error = True

    return {
        'error': error,
        'error_message': error_message,
        'periodo_fin': period_fi,
        'periodo_ini': period_ini,
        'otherstats': otherstats,
        'jobs': jobs,
        'stats': {            
            'max_time': results.max_time if results is not None else  "",
            'max_fail': results.max_fail if results is not None else  "",
            'queued': results.queued if results is not None else  "",
            'run': results.run if results is not None else  "",
            'failed_jobs': results.failed_jobs if results is not None else  "",
            'fail_queued': results.fail_queued if results is not None else  "",
            'fail_run': results.fail_run if results is not None else  "",
            'threshold': results.threshold if results is not None else  "",
            'totals': results._totals if results is not None else "",
        }
    }


def get_experiment_data(expid):
    """
    Get data of the experiment. Depends on the run cron of autosubmit.
    """
    path = 'Not found'
    conf_contents = ()
    tmp_contents = list()
    pkl_contents = list()
    plot_contents = list()
    status_contents = list()
    owner_id = 0
    owner_name = 'Not found'
    datetime_lastAccess = '1900-01-01 00:00:00'
    datetime_lastMod = '1900-01-01 00:00:00'
    error_message = 'None'
    contents = list()
    description = 'None'
    version = 'None'
    model = 'None'
    branch = 'None'
    hpc = 'None'
    updateTime = 10
    error = False
    running = False
    pkl_timestamp = 1000000
    reading = ""

    BasicConfig.read()
    path = BasicConfig.LOCAL_ROOT_DIR+'/'+expid

    if (os.path.exists(path)):
        try:  
            as_conf = AutosubmitConfig(expid, BasicConfig, ConfigParserFactory())
            if as_conf.check_conf_files() == False:
                raise Exception('Autosubmit GUI might not have permission to access necessary configuration files.')
            updateTime = as_conf.get_safetysleeptime()
            project_type = as_conf.get_project_type()
            if project_type != "none":
                if not as_conf.check_proj():
                    raise Exception('Autosubmit GUI might not have permission to access necessary configuration files.')
            if (as_conf.get_svn_project_url()):
                model = as_conf.get_svn_project_url()
                branch = as_conf.get_svn_project_url()
            else:
                model = as_conf.get_git_project_origin()
                branch = as_conf.get_git_project_branch()
            if model is "":
                model = "Not Found"
            if branch is "":
                branch = "Not Found"

            submitter = Autosubmit._get_submitter(as_conf)
            hpc = as_conf.get_platform()
            
            resultBase = db_common.get_experiment_by_id(expid)          
            description = resultBase['description']
            version = resultBase['version']
            path_pkl = path + '/pkl'
            path_conf = path + '/conf'
            path_tmp = path + '/tmp'
            path_plot = path + '/plot'
            path_status = path + '/status'
            # print(path)
            contents = os.listdir(path)    
            # Getting pkl timestamp
            readingpkl = os.popen('ls -t ' + path_pkl + ' | grep ".pkl"').read() if (os.path.exists(path_pkl)) else ""
            #print(readingpkl)
            #pkl_contents = os.listdir(path_pkl) if (os.path.exists(path_pkl)) else list()
            #pkl_files = list(filter(lambda x: x.endswith(expid + '.pkl'), pkl_contents))
            if len(readingpkl) > 0:
                #print(readingpkl)
                pkl_file_name = readingpkl.split()[0]
                #print(pkl_file_name)
                current_stat = os.stat(path_pkl+'/'+pkl_file_name)
                pkl_timestamp = int(current_stat.st_mtime)
                #print(int(pkl_timestamp))

            status = os.stat(path)
            owner_id = status.st_uid
            owner_name = os.popen('id -nu ' + str(owner_id)).read().strip()     

            time_lastAccess = int(status.st_atime)
            datetime_lastAccess = datetime.datetime.utcfromtimestamp(time_lastAccess).strftime('%Y-%m-%d %H:%M:%S')
            time_lastMod = int(status.st_mtime)
            datetime_lastMod = datetime.datetime.utcfromtimestamp(time_lastMod).strftime('%Y-%m-%d %H:%M:%S')    
            size = str(status.st_size) + ' bytes' # get_size(path) # 

        except Exception as e:
            error_message = str(e)
            error = True
    else:
        error_message = str(expid) + " not found in " + str(BasicConfig.LOCAL_ROOT_DIR)

    result = {
     'expid' : expid,
     'path': path, 
     'contents': contents,           
     'contents_conf': conf_contents,
     'contents_tmp': tmp_contents, 
     'contents_pkl': pkl_contents,
     'contents_plot': plot_contents,
     'contents_status': status_contents,
    'owner_id': owner_id, 
    'owner': owner_name, 
    'time_last_access': datetime_lastAccess, 
    'time_last_mod': datetime_lastMod,
    'error_message': error_message,
    'description': description,
    'version' : version,
    'model': model,
    'branch': branch,
    'hpc': hpc,
    'updateTime': updateTime,
    'error': error,
    'running': running,
    'pkl_timestamp': pkl_timestamp}

    return result

def _is_exp_running(expid, time_condition = 300):
    """
    Tests if experiment is running
    :param expid: Experiment name
    :param time_condition: Time constraint, 120 by default. Represents max seconds before an experiment is considered as NOT RUNNING
    :return: (error (true if error is found, false otherwise), error_message, is_running (true if running, false otherwise), timediff)
    :rtype: tuple (bool, string, bool, int)
    """
    is_running = False
    error = False
    error_message = ""
    timediff = 0
    try:
        BasicConfig.read()
        pathlog = BasicConfig.LOCAL_ROOT_DIR + '/' + expid + '/' + BasicConfig.LOCAL_TMP_DIR + '/' + BasicConfig.LOCAL_ASLOG_DIR
        reading = os.popen('ls -t ' + pathlog + ' | grep "run.log"').read() if (os.path.exists(pathlog)) else ""
        if (reading) and len(reading) > 0:
            log_file_name = reading.split()[0]
            current_stat = os.stat(pathlog+'/'+log_file_name)
            timest = current_stat.st_mtime
            #system_stat = os.stat(BasicConfig.LOCAL_ROOT_DIR)
            timesys = time.time()
            timediff = int(timesys - timest)
            # print(timediff)
            if (timediff < time_condition):
                is_running = True
            else:
                is_running = False            
            return (error, error_message, is_running, timediff)
            
        
        pathlog = BasicConfig.LOCAL_ROOT_DIR + '/' + expid + '/' + BasicConfig.LOCAL_TMP_DIR
        reading = os.popen('ls -t ' + pathlog + ' | grep "run.log"').read() if (os.path.exists(pathlog)) else ""
        if (reading) and len(reading) > 0:
            log_file_name = reading.split()[0]
            current_stat = os.stat(pathlog+'/'+log_file_name)
            timest = current_stat.st_mtime
            #system_stat = os.stat(BasicConfig.LOCAL_ROOT_DIR)
            timesys = time.time()
            timediff = int(timesys - timest)
            print(timediff)
            if (timediff < time_condition):
                is_running = True
            else:
                is_running = False
            # print("Result done")
            return (error, error_message, is_running, timediff)

    except Exception as ex:
        error = True
        is_running = False
        timediff = time_condition
        error_message = str(ex)
        print("Error in test: " + error_message)
        return (error, error_message, is_running, timediff)

def test_run(expid):
    """
    Tests if experiment is running
    """
    running = False
    error = False
    error_message = ""   
    timediff = 0

    try:
        error, error_message, running, timediff  = _is_exp_running(expid, time_condition=120)
    except Exception as ex:
        error = True
        error_message = str(ex)

    return { 'running' : running,
            'error' : error,
            'error_message' : error_message }


def get_experiment_run(expid):
    """
    Gets log content
    """
    # Initializing results:
    log_file_name = ""
    found = False
    log_file_lastmodified = ""
    timest = ""
    error = False
    error_message = ""    
    logcontent = []
    reading = ""

    try:
        BasicConfig.read()
        path = BasicConfig.LOCAL_ROOT_DIR + '/' + expid + '/' + BasicConfig.LOCAL_TMP_DIR + '/' + BasicConfig.LOCAL_ASLOG_DIR
        print(path)
        reading = os.popen('ls -t ' + path + ' | grep "run.log"').read() if (os.path.exists(path)) else ""

        # Finding log files    
        if len(reading) == 0:
            path = BasicConfig.LOCAL_ROOT_DIR + '/' + expid + '/' + BasicConfig.LOCAL_TMP_DIR
            reading = os.popen('ls -t ' + path + ' | grep "run.log"').read() if (os.path.exists(path)) else ""
            
        if len(reading) > 0:
            # run_logs.sort(reverse = True)
            log_file_name = reading.split()[0]
            print(log_file_name)
            current_stat = os.stat(path+'/'+log_file_name)
            timest = current_stat.st_mtime
            log_file_lastmodified = datetime.datetime.utcfromtimestamp(timest).strftime('%Y-%m-%d %H:%M:%S')
            found = True
            # line = subprocess.check_output(['tail', '-50', path+'/'+log_file_name])
            request = 'tail -50 ' + path + '/' + log_file_name
            print(request)
            last50 = os.popen(request)

            i = 0
            for item in last50.readlines():
                logcontent.append({'index': i, 'content': item[0:-1] })
                i+=1
    except Exception as e:
        error = True
        error_message = str(e)
        
    return { 
        'logfile' : log_file_name, 
        'found': found, 
        'lastModified': log_file_lastmodified, 
        'timeStamp': timest,
        'error': error,
        'error_message': error_message,
        'logcontent': logcontent }

def get_experiment_pkl(expid, modTimestamp):
    pkl_file_name = ""
    error = False
    error_message = ""
    has_changed = False
    pkl_content = list()
    timest = 1000000
    try:
        BasicConfig.read()
        path = BasicConfig.LOCAL_ROOT_DIR + '/' + expid + '/pkl'
        reading = os.popen('ls -t ' + path + ' | grep ".pkl"').read() if (os.path.exists(path)) else ""
        # pkl_contents = os.listdir(path) if (os.path.exists(path)) else list()
        # pkl_files = list(filter(lambda x: x.endswith(expid + '.pkl'), pkl_contents))
        path_to_logs = os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, "tmp", "LOG_" + expid)
        if len(reading) > 0:
            pkl_file_name = reading.split()[0]
            current_stat = os.stat(path+'/'+pkl_file_name)
            timest = int(current_stat.st_mtime)
            #print(timest)
            if timest - int(modTimestamp) > 0:
                has_changed = True
        else:
            raise Exception('Empty pkl directory.')

        if has_changed == True:
            # Dealing with packages
            job_to_package = dict()
            package_to_jobs = dict()
            package_to_package_id = dict()
            job_to_package, package_to_jobs, package_to_package_id = JobList.retrieve_packages(BasicConfig, expid)  

            path_pkl = os.path.join(path, pkl_file_name)
            print(path_pkl)
            if os.path.exists(path_pkl):
                fd = open(path_pkl, 'r')            
                for item in pickle.load(fd):
                    pkl_content.append({'name': item[0],
                    'id': item[1],
                    'status_code': item[2],
                    'dashed': True if item[0] in list(job_to_package.keys()) else False,
                    'shape': 'square' if item[0] in list(job_to_package.keys()) else 'dot',
                    'package': job_to_package[item[0]] if item[0] in list(job_to_package.keys()) else '',
                    'status': str(Status.VALUE_TO_KEY[item[2]]),
                    'status_color': Monitor.color_status(item[2]),
                    'out': path_to_logs + "/" + (str(item[8]) if len(item) >= 9 else ""),                
                    'err': path_to_logs + "/" + (str(item[9]) if len(item) >= 10 else ""),
                    'priority': item[3]})
            else:
                raise Exception('File {0} does not exist'.format(path))

    except Exception as e:
        error = True
        error_message = str(e)

    return {
        'pkl_file_name' : pkl_file_name,
        'error' : error,
        'error_message' : error_message,
        'has_changed' : has_changed,
        'pkl_content' : pkl_content,
        'pkl_timestamp' : timest,
        'packages': package_to_jobs if has_changed == True else {},
    }

def get_experiment_tree_pkl(expid, modTimestamp):
    pkl_file_name = ""
    error = False
    error_message = ""
    #has_changed = False
    # For Refresh purposes, it always changes
    has_changed = True
    pkl_content = list()
    timest = 1000000
    source = JobList.get_sourcetag()
    target = JobList.get_targettag()
    sync = JobList.get_synctag()
    check_mark = JobList.get_checkmark()
    try:
        BasicConfig.read()
        path = BasicConfig.LOCAL_ROOT_DIR + '/' + expid + '/pkl'
        path_to_logs = os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, "tmp", "LOG_" + expid)
        tmp_path = os.path.join(BasicConfig.LOCAL_ROOT_DIR,expid, BasicConfig.LOCAL_TMP_DIR)
        reading = os.popen('ls -t ' + path + ' | grep ".pkl"').read() if (os.path.exists(path)) else ""
        # pkl_contents = os.listdir(path) if (os.path.exists(path)) else list()
        # pkl_files = list(filter(lambda x: x.endswith(expid + '.pkl'), pkl_contents))
        # Try to get packages
        job_to_package = dict()
        package_to_jobs = dict()
        package_to_package_id = dict()

        job_to_package, package_to_jobs, package_to_package_id = JobList.retrieve_packages(BasicConfig, expid)  

        if len(reading) > 0:
            pkl_file_name = reading.split()[0]
            current_stat = os.stat(path+'/'+pkl_file_name)
            timest = int(current_stat.st_mtime)
            #print(timest)
            if timest - int(modTimestamp) > 0:
                has_changed = True
        else:
            raise Exception('Empty pkl directory.')

        if has_changed == True:
            path_pkl = os.path.join(path, pkl_file_name)
            print(path_pkl)
            if os.path.exists(path_pkl):
                fd = open(path_pkl, 'r')            
                for item in pickle.load(fd):
                    status_code = item[2]
                    job_name = item[0]
                    status_color = Monitor.color_status(status_code)
                    status_text = str(Status.VALUE_TO_KEY[status_code])
                    wrapper_tag = ""
                    wrapper_id = 0
                    wrapper_name = ""
                    if job_name in job_to_package.keys():
                        wrapper_name  = job_to_package[job_name]
                        wrapper_id = package_to_package_id[job_to_package[job_name]]
                        wrapper_tag = " <span class='badge' style='background-color:#94b8b8'>Wrapped " + wrapper_id + "</span>"
                    minutes, running_text = JobList.retrieve_times(status_code, job_name, tmp_path)                    
                    pkl_content.append({'name': job_name,
                    'id': item[1],
                    'status_code': status_code,
                    'minutes': minutes if minutes > 0 else -1,
                    'running_text': running_text if minutes > 0 else "",
                    'status': status_text,
                    'status_color': status_color,
                    'wrapper': wrapper_name,
                    'wrapper_tag': wrapper_tag,
                    'wrapper_id': wrapper_id,
                    'out': path_to_logs + "/" + (str(item[8]) if len(item) >= 9 else ""),                
                    'err': path_to_logs + "/" + (str(item[9]) if len(item) >= 10 else ""),
                    'title': Job.getTitle(job_name, status_color, status_text) + ((" ~ " + running_text) if minutes > 0 else ""),
                    'priority': item[3]})
            else:
                raise Exception('File {0} does not exist'.format(path))

    except Exception as e:
        error = True
        error_message = str(e)

    return {
        'pkl_file_name' : pkl_file_name,
        'error' : error,
        'error_message' : error_message,
        'has_changed' : has_changed,
        'pkl_content' : pkl_content,
        'packages' : list(package_to_jobs.keys()),
        'pkl_timestamp' : timest,
        'source_tag' : source,
        'target_tag' : target,
        'sync_tag' : sync,
        'check_mark' : check_mark,
    }


def get_experiment_graph(expid, layout = 'standard', grouped = 'none'):
    """
    Gets graph representation
    """
    base_list = dict()
    pkl_timestamp = 10000000
    try:
        notransitive = False
        print("Received " + str(expid) + " ~ " + layout + " ~ " + grouped)
        BasicConfig.read()
        exp_path = os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid)
        tmp_path = os.path.join(exp_path, BasicConfig.LOCAL_TMP_DIR)

        as_conf = AutosubmitConfig(
            expid, BasicConfig, ConfigParserFactory())

        if not as_conf.check_conf_files():
            # Log.critical('Can not run with invalid configuration')
            raise Exception('Autosubmit GUI might not have permission to access necessary configuration files.')


        job_list = Autosubmit.load_job_list(
            expid, as_conf, notransitive=notransitive)     
        

        job_list.sort_by_id()      
        base_list = job_list.get_graph_representation(BasicConfig, layout, grouped)
    except Exception as e:
        return {'nodes': [], 
        'edges': [], 
        'fake_edges': [],
        'groups': [],
        'groups_data': {},
        'error': True, 
        'error_message': str(e), 
        'graphviz' : False,
        'max_children' : 0, 
        'max_parents' : 0,
        'total_jobs' : 0,
        'pkl_timestamp': 0}

    base_list['error'] = False
    base_list['error_message'] = 'None'
    base_list['pkl_timestamp'] = pkl_timestamp
    return base_list

def get_experiment_tree_structured(expid):
    """
    Current version of the tree visualization algorithm
    """
    base_list = dict()
    pkl_timestamp = 10000000
    try:
        notransitive = False
        print("Received Tree Request " + str(expid))
        BasicConfig.read()
        exp_path = os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid)
        tmp_path = os.path.join(exp_path, BasicConfig.LOCAL_TMP_DIR)
        as_conf = AutosubmitConfig(
            expid, BasicConfig, ConfigParserFactory())
        if not as_conf.check_conf_files():
            # Log.critical('Can not run with invalid configuration')
            raise Exception('Autosubmit GUI might not have permission to access necessary configuration files.')

        job_list = Autosubmit.load_job_list(
            expid, as_conf, notransitive=notransitive) 
        #print("Job List loaded")    
        #job_list.sort_by_id()      
        #base_list['tree'] = job_list.get_tree_representation().traverse_format()
        
        tree_structure, current_collection, reference = job_list.get_tree_structured(BasicConfig)
        base_list['tree'] = tree_structure
        base_list['jobs'] = current_collection
        base_list['total'] = len(current_collection)
        base_list['reference'] = reference
    except Exception as e:
        return {'tree': [], 'jobs': [], 'total': 0, 'reference': [], 'error': True, 'error_message': str(e), 'pkl_timestamp': 0}
    base_list['error'] = False
    base_list['error_message'] = 'None'
    base_list['pkl_timestamp'] = pkl_timestamp
    return base_list


def get_experiment_graph_format_test(expid):
    """
    Some testing
    """    
    base_list = dict()
    pkl_timestamp = 10000000
    try:
        notransitive = False
        print("Received " + str(expid))
        BasicConfig.read()
        exp_path = os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid)
        tmp_path = os.path.join(exp_path, BasicConfig.LOCAL_TMP_DIR)
        as_conf = AutosubmitConfig(
            expid, BasicConfig, ConfigParserFactory())
        if not as_conf.check_conf_files():
            # Log.critical('Can not run with invalid configuration')
            raise Exception('Autosubmit GUI might not have permission to access necessary configuration files.')


        job_list = Autosubmit.load_job_list(
            expid, as_conf, notransitive=notransitive)     

        job_list.sort_by_id()      
        base_list = job_list.get_graph_representation(BasicConfig)
    except Exception as e:
        return {'nodes': [], 
        'edges': [], 
        'error': True, 
        'error_message': str(e), 
        'graphviz' : False,
        'max_children' : 0, 
        'max_parents' : 0,
        'total_jobs' : 0,
        'pkl_timestamp': 0}
    name_to_id = dict()
    #name_to_weight = dict()
    list_nodes = list()
    list_edges = list()
    i = 0
    with open('/home/Earth/wuruchi/Documents/Personal/spectralgraph/data/graph_' + expid +'.txt', 'w') as the_file:
        for item in base_list['nodes']:
            #the_file.write(str(i) + "\n")
            name_to_id[item['id']] = i            
            list_nodes.append(i)
            i += 1
        for item in base_list['edges']:            
            the_file.write(str(name_to_id[item['from']]) + " " + str(name_to_id[item['to']]) + " " + ("10" if item['is_wrapper'] == True else "1") + "\n")
            list_edges.append((name_to_id[item['from']], name_to_id[item['to']]))
        for item in base_list['fake_edges']:
            the_file.write(str(name_to_id[item['from']]) + " " + str(name_to_id[item['to']]) + " " + ("10" if item['is_wrapper'] == True else "1") + "\n")
            list_edges.append((name_to_id[item['from']], name_to_id[item['to']]))
    return list_nodes, list_edges

def process_completed_times(time_condition=0):
    """
    Tests for completed jobs and updates their times in database    
    """
    try:
        BasicConfig.read()
        path = BasicConfig.LOCAL_ROOT_DIR
        experiments = os.popen('ls -t ' + path).read() if (os.path.exists(path)) else ""
        db_file = os.path.join(path,"ecearth.db")
        conn = DbRequests.create_connection(db_file)
        current_table = DbRequests.prepare_completed_times_db(conn)
        experiments = experiments.split()
        total_process = len(experiments)
        counter = 0
        start_time_operation = time.time()
        for expid in experiments:            
            counter += 1
            pkl_path = os.path.join(path, expid, "pkl")
            pkl_found = os.popen('ls -t ' + pkl_path + ' | grep ".pkl"').read() if (os.path.exists(pkl_path)) else ""
            if (len(pkl_found) > 0):                
                full_path = pkl_path+'/'+(pkl_found[:-1])
                if (os.path.exists(full_path)):
                    # Pkl exists 
                    print("Processing " + str(counter) + " of " + str(total_process))
                    start_time_local = time.time()
                    current_stat = os.stat(full_path)
                    timest = int(current_stat.st_mtime)
                    if expid not in current_table.keys():
                        # Pkl exists but is not registered in the table
                        # INSERT

                        current_id = _process_pkl_insert_times(conn, expid, full_path, timest, BasicConfig)                        
                        #return
                    else:
                        exp_id, created, modified, total_jobs, completed_jobs = current_table[expid]                        
                        time_diff = int(timest - modified)
                        if time_diff > time_condition:
                            print("UPDATE SHOULD BE HERE")
                            # The pkl is ahead of the register in the database
                            # UPDATE
                    total_seconds_local = time.time() - start_time_local
                    print("Seconds spent: " + str(total_seconds_local))
            else:
                print("Pkl " + pkl_path + " not found.")
                print("Processing " + str(counter) + " of " + str(total_process))
        total_seconds = time.time() - start_time_operation
        print("Minutes spent in total: " + str(total_seconds/60)) 
    except Exception as ex:
        print(str(ex))

def _process_pkl_update_times(conn, expid, path_pkl, timest_pkl, BasicConfig, exp_id, total_jobs, completed_jobs):
    """
    Updates register in job_times and experiment_times for the given experiment
    """
    tmp_path = os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, BasicConfig.LOCAL_TMP_DIR)
    job_times_db = dict()
    total_jobs = 0
    completed_jobs = 0
    fd = open(path_pkl, 'r')                
    for item in pickle.load(fd):            
        total_jobs += 1
        status_code = int(item[2])
        job_name = item[0]
        minutes = 0
        status_text = str(Status.VALUE_TO_KEY[status_code])
        # if (status_code == Status.COMPLETED):
        #     completed_jobs += 1    
        #     minutes, running_text = JobList.retrieve_times(status_code, job_name, tmp_path, make_exception=True)    
        #     print(job_name + " ~ " + status_text + " in " + tmp_path)                 
        # job_times[job_name] = (status_code, minutes)

    experiment_times_detail = DbRequests.get_times_detail(conn, expid)




def _process_pkl_insert_times(conn, expid, path_pkl, timest_pkl, BasicConfig):
    """
    Process Pkl and insert into database
    """
    # if os.path.exists(path_pkl):
    # DbRequests._get_id_db(conn, expid)    
    tmp_path = os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, BasicConfig.LOCAL_TMP_DIR)
    job_times = dict() # Key: job_name
    total_jobs = 0
    completed_jobs = 0
    fd = open(path_pkl, 'r')      
    try:
        for item in pickle.load(fd):            
            total_jobs += 1
            status_code = int(item[2])
            job_name = item[0]
            submit_time = ""
            start_time = ""
            finish_time = ""
            status_text = str(Status.VALUE_TO_KEY[status_code])
            if (status_code == Status.COMPLETED):
                completed_jobs += 1    
                submit_time, start_time, finish_time, status_text = JobList._job_running_check(status_code, job_name, tmp_path) # retrieve_times(status_code, job_name, tmp_path, make_exception=True)                    
            job_times[job_name] = (submit_time, start_time, finish_time, status_text)
        # Insert header
        current_id = DbRequests.insert_experiment_times_header(conn, expid, timest_pkl, total_jobs, completed_jobs)
        if(current_id > 0):
            # Insert detail
            for job_name in job_times.keys():
                # Detail
                submit_time, start_time, finish_time, status_text = job_times[job_name]
                DbRequests.create_job_times(conn, 
                current_id, 
                expid, 
                job_name, 
                int(timest_pkl), 
                int(time.mktime(submit_time.timetuple())) if len(str(submit_time)) > 0 else 0, 
                int(time.mktime(start_time.timetuple())) if len(str(start_time)) > 0 else 0, 
                int(time.mktime(finish_time.timetuple())) if len(str(finish_time)) > 0 else 0, 
                status_text)
        else:
            raise Exception('Header could not be inserted: ' + str(expid))
        return current_id
    except Exception as ex:
        print(str(ex))
        return 0
    

def update_running_experiments(time_condition = 300):
    """
    Tests if an experiment is running and updates database accordingly
    """
    experiment_to_modified_ts = dict()
    try:
        BasicConfig.read()
        path = BasicConfig.LOCAL_ROOT_DIR
        readingpkl = os.popen('ls -t ' + path).read() if (os.path.exists(path)) else ""        
        db_file = os.path.join(path,"ecearth.db")
        conn = DbRequests.create_connection(db_file)
        current_table = DbRequests.prepare_status_db(conn)       
        readingpkl = readingpkl.split()
        for item in readingpkl:
            pkl_path = os.path.join(path, item, "pkl")
            pkl_found = os.popen('ls -t ' + pkl_path + ' | grep ".pkl"').read() if (os.path.exists(pkl_path)) else ""
            if (len(pkl_found) > 0):
                full_path = pkl_path+'/'+(pkl_found[:-1])
                if (os.path.exists(full_path)):
                    current_stat = os.stat(full_path)
                    timest = int(current_stat.st_mtime)
                    timesys = time.time()                
                    # experiment_to_modified_ts[item] = timest
                    time_diff = int(timesys - timest)
                    if (time_diff < time_condition):
                        # print("Experiment : " + item + " is RUNNING " + str(time_diff))
                        experiment_to_modified_ts[item] = time_diff
                        if item in current_table.keys():
                            # print(item + " is already in table")
                            # UPDATE RUNNING
                            _exp_id, _status, _seconds = current_table[item]
                            # print("Status in db " + _status)
                            if _status != "RUNNING":
                                # print("UPDATING " + item + " TO RUNNING")
                                DbRequests.update_exp_status(conn, item, "RUNNING", time_diff)
                        else:
                            # INSERT
                            # print("Insert new experiment : " +  str(item))
                            current_id = DbRequests.insert_experiment_status(conn, item, time_diff)
                            current_table[item] = (current_id, 'RUNNING', time_diff)
        
        for expid in current_table.keys():
            exp_id, status, seconds = current_table[expid]
            if status == "RUNNING" and expid not in experiment_to_modified_ts.keys():
                # Perform exhaustive check
                # print("Performing exhaustive test.")
                error, error_message, is_running, timediff = _is_exp_running(expid)
                # UPDATE NOT RUNNING
                if (is_running == False):
                    # print("Update NOT RUNNING for " + expid)
                    DbRequests.update_exp_status(conn, expid, "NOT RUNNING", timediff)

        # populate_db(db_file, experiment_to_modified_ts)

    except Exception as e:
        print(e)
